@@ -64,7 +64,8 @@ typedef enum {
 	TRACKER_DB_QUERY_ERROR,
 	TRACKER_DB_INTERRUPTED,
 	TRACKER_DB_OPEN_ERROR,
-	TRACKER_DB_NO_SPACE
+	TRACKER_DB_NO_SPACE,
+	TRACKER_DB_CONSTRAINT,
 } TrackerDBInterfaceError;
 
 typedef enum {
@@ -97,7 +98,11 @@ void                    tracker_db_interface_set_user_data           (TrackerDBI
 gpointer                tracker_db_interface_get_user_data           (TrackerDBInterface         *interface);
 
 /* Functions to create queries/procedures */
-TrackerDBStatement *    tracker_db_interface_create_statement        (TrackerDBInterface          *interface,
+TrackerDBStatement *    tracker_db_interface_create_statement        (TrackerDBInterface           *db_interface,
+                                                                      TrackerDBStatementCacheType   cache_type,
+                                                                      GError                      **error,
+                                                                      const gchar                  *query);
+TrackerDBStatement *    tracker_db_interface_create_vstatement       (TrackerDBInterface          *interface,
                                                                       TrackerDBStatementCacheType  cache_type,
                                                                       GError                     **error,
                                                                       const gchar                 *query,
