@@ -198,13 +198,13 @@ db_set_params (TrackerDBInterface   *iface,
 	tracker_db_interface_execute_query (iface, NULL, "PRAGMA \"%s\".auto_vacuum = 0", database);
 
 	if (enable_wal) {
-		stmt = tracker_db_interface_create_statement (iface, TRACKER_DB_STATEMENT_CACHE_TYPE_NONE,
-		                                              &internal_error,
-		                                              "PRAGMA \"%s\".journal_mode = WAL", database);
+		stmt = tracker_db_interface_create_vstatement (iface, TRACKER_DB_STATEMENT_CACHE_TYPE_NONE,
+		                                               &internal_error,
+		                                               "PRAGMA \"%s\".journal_mode = WAL", database);
 
 		if (internal_error) {
-			g_info ("Can't set journal mode to WAL: '%s'",
-			        internal_error->message);
+			g_debug ("Can't set journal mode to WAL: '%s'",
+			         internal_error->message);
 			g_propagate_error (error, internal_error);
 		} else {
 			TrackerDBCursor *cursor;
@@ -324,9 +324,9 @@ tracker_db_manager_update_version (TrackerDBManager *db_manager)
 	GError *error = NULL;
 
 	iface = tracker_db_manager_get_writable_db_interface (db_manager);
-	stmt = tracker_db_interface_create_statement (iface, TRACKER_DB_STATEMENT_CACHE_TYPE_NONE,
-	                                              &error, "PRAGMA user_version = %d",
-						      TRACKER_DB_VERSION_NOW);
+	stmt = tracker_db_interface_create_vstatement (iface, TRACKER_DB_STATEMENT_CACHE_TYPE_NONE,
+	                                               &error, "PRAGMA user_version = %d",
+	                                               TRACKER_DB_VERSION_NOW);
 	if (stmt) {
 		tracker_db_statement_execute (stmt, &error);
 		g_object_unref (stmt);
@@ -392,7 +392,7 @@ tracker_db_manager_locale_changed (TrackerDBManager  *db_manager,
 		             current_locale);
 		changed = TRUE;
 	} else {
-		g_info ("Current and DB locales match: '%s'", db_locale);
+		g_debug ("Current and DB locales match: '%s'", db_locale);
 		changed = FALSE;
 	}
 
@@ -409,7 +409,7 @@ tracker_db_manager_set_current_locale (TrackerDBManager *db_manager)
 
 	/* Get current collation locale */
 	current_locale = tracker_locale_get (TRACKER_LOCALE_COLLATE);
-	g_info ("Saving DB locale as: '%s'", current_locale);
+	g_debug ("Saving DB locale as: '%s'", current_locale);
 	db_set_locale (db_manager, current_locale);
 	g_free (current_locale);
 }
@@ -663,7 +663,7 @@ tracker_db_manager_new (TrackerDBManagerFlags   flags,
 				/* Size is 1 when using echo > file.db, none of our databases
 				 * are only one byte in size even initually. */
 				if (size <= 1) {
-					g_info ("Database is corrupt: size is 1 byte or less.");
+					g_debug ("Database is corrupt: size is 1 byte or less.");
 					return FALSE;
 				}
 
@@ -819,7 +819,8 @@ TrackerDBInterface *
 tracker_db_manager_get_db_interface (TrackerDBManager *db_manager)
 {
 	GError *internal_error = NULL;
-	TrackerDBInterface *interface;
+	TrackerDBInterface *interface = NULL;
+	guint len, i;
 
 	/* The interfaces never actually leave the async queue,
 	 * we use it as a thread synchronized LRU, which doesn't
@@ -830,19 +831,30 @@ tracker_db_manager_get_db_interface (TrackerDBManager *db_manager)
 	 * in the interface lock).
 	 */
 	g_async_queue_lock (db_manager->interfaces);
-	interface = g_async_queue_try_pop_unlocked (db_manager->interfaces);
+	len = g_async_queue_length_unlocked (db_manager->interfaces);
 
-	if (interface && tracker_db_interface_get_is_used (interface) &&
-	    g_async_queue_length_unlocked (db_manager->interfaces) < MAX_INTERFACES) {
-		/* Put it back and go at creating a new one */
-		g_async_queue_push_front_unlocked (db_manager->interfaces, interface);
+	/* 1st. Find a free interface */
+	for (i = 0; i < len; i++) {
+		interface = g_async_queue_try_pop_unlocked (db_manager->interfaces);
+
+		if (!interface)
+			break;
+		if (!tracker_db_interface_get_is_used (interface))
+			break;
+
+		g_async_queue_push_unlocked (db_manager->interfaces, interface);
 		interface = NULL;
 	}
 
+	/* 2nd. If no more interfaces can be created, pick one */
+	if (!interface && len >= MAX_INTERFACES) {
+		interface = g_async_queue_try_pop_unlocked (db_manager->interfaces);
+	}
+
 	if (interface) {
 		g_signal_emit (db_manager, signals[UPDATE_INTERFACE], 0, interface);
 	} else {
-		/* Create a new one to satisfy the request */
+		/* 3rd. Create a new interface to satisfy the request */
 		interface = tracker_db_manager_create_db_interface (db_manager,
 		                                                    TRUE, &internal_error);
 
