@@ -50,6 +50,7 @@
 #include "tracker-private.h"
 #include "tracker-sparql-enum-types.h"
 #include <libtracker-common/tracker-common.h>
+#include <direct/tracker-direct.h>
 
 typedef struct _TrackerNotifierPrivate TrackerNotifierPrivate;
 typedef struct _TrackerNotifierSubscription TrackerNotifierSubscription;
@@ -70,7 +71,9 @@ struct _TrackerNotifierPrivate {
 	GHashTable *subscriptions; /* guint -> TrackerNotifierSubscription */
 	GCancellable *cancellable;
 	TrackerSparqlStatement *local_statement;
+	GAsyncQueue *queue;
 	gint n_local_statement_slots;
+	gboolean querying;
 	GMutex mutex;
 };
 
@@ -454,6 +457,8 @@ handle_cursor (GTask        *task,
 {
 	TrackerNotifierEventCache *cache = task_data;
 	TrackerSparqlCursor *cursor = source_object;
+	TrackerNotifier *notifier = cache->notifier;
+	TrackerNotifierPrivate *priv = tracker_notifier_get_instance_private (notifier);
 	TrackerNotifierEvent *event;
 	GSequenceIter *iter;
 	gint64 id;
@@ -486,9 +491,19 @@ handle_cursor (GTask        *task,
 	cache->first = iter;
 
 	if (g_sequence_iter_is_end (cache->first)) {
+		TrackerNotifierEventCache *next;
+
 		tracker_notifier_emit_events_in_idle (cache);
+
+		g_async_queue_lock (priv->queue);
+		next = g_async_queue_try_pop_unlocked (priv->queue);
+		if (next)
+			tracker_notifier_query_extra_info (notifier, next);
+		else
+			priv->querying = FALSE;
+		g_async_queue_unlock (priv->queue);
 	} else {
-		tracker_notifier_query_extra_info (cache->notifier, cache);
+		tracker_notifier_query_extra_info (notifier, cache);
 	}
 
 	g_task_return_boolean (task, TRUE);
@@ -609,6 +624,7 @@ void
 _tracker_notifier_event_cache_flush_events (TrackerNotifierEventCache *cache)
 {
 	TrackerNotifier *notifier = cache->notifier;
+	TrackerNotifierPrivate *priv = tracker_notifier_get_instance_private (notifier);
 
 	if (g_sequence_is_empty (cache->sequence)) {
 		_tracker_notifier_event_cache_free (cache);
@@ -616,7 +632,15 @@ _tracker_notifier_event_cache_flush_events (TrackerNotifierEventCache *cache)
 	}
 
 	cache->first = g_sequence_get_begin_iter (cache->sequence);
-	tracker_notifier_query_extra_info (notifier, cache);
+
+	g_async_queue_lock (priv->queue);
+	if (priv->querying) {
+		g_async_queue_push_unlocked (priv->queue, cache);
+	} else {
+		priv->querying = TRUE;
+		tracker_notifier_query_extra_info (notifier, cache);
+	}
+	g_async_queue_unlock (priv->queue);
 }
 
 static void
@@ -691,6 +715,7 @@ tracker_notifier_finalize (GObject *object)
 	g_cancellable_cancel (priv->cancellable);
 	g_clear_object (&priv->cancellable);
 	g_clear_object (&priv->local_statement);
+	g_async_queue_unref (priv->queue);
 
 	if (priv->connection)
 		g_object_unref (priv->connection);
@@ -755,6 +780,7 @@ tracker_notifier_init (TrackerNotifier *notifier)
 	priv->subscriptions = g_hash_table_new_full (NULL, NULL, NULL,
 	                                             (GDestroyNotify) tracker_notifier_subscription_free);
 	priv->cancellable = g_cancellable_new ();
+	priv->queue = g_async_queue_new ();
 }
 
 /**
@@ -786,6 +812,7 @@ tracker_notifier_signal_subscribe (TrackerNotifier *notifier,
 {
 	TrackerNotifierSubscription *subscription;
 	TrackerNotifierPrivate *priv;
+	gchar *dbus_name = NULL, *dbus_path = NULL, *full_graph = NULL;
 
 	g_return_val_if_fail (TRACKER_IS_NOTIFIER (notifier), 0);
 	g_return_val_if_fail (G_IS_DBUS_CONNECTION (connection), 0);
@@ -796,16 +823,32 @@ tracker_notifier_signal_subscribe (TrackerNotifier *notifier,
 	if (!object_path)
 		object_path = DEFAULT_OBJECT_PATH;
 
+	if (graph) {
+		TrackerNamespaceManager *namespaces;
+
+		namespaces = tracker_sparql_connection_get_namespace_manager (priv->connection);
+		if (namespaces) {
+			full_graph = tracker_namespace_manager_expand_uri (namespaces,
+			                                                   graph);
+		}
+	}
+
+	tracker_sparql_connection_lookup_dbus_service (priv->connection,
+	                                               service,
+	                                               object_path,
+	                                               &dbus_name,
+	                                               &dbus_path);
+
 	subscription = tracker_notifier_subscription_new (notifier, connection,
 	                                                  service, object_path);
 
 	subscription->handler_id =
 		g_dbus_connection_signal_subscribe (connection,
-		                                    service,
+		                                    dbus_name ? dbus_name : service,
 		                                    "org.freedesktop.Tracker3.Endpoint",
 		                                    "GraphUpdated",
-		                                    object_path,
-		                                    graph,
+		                                    dbus_path ? dbus_path : object_path,
+		                                    full_graph ? full_graph : graph,
 		                                    G_DBUS_SIGNAL_FLAGS_NONE,
 		                                    graph_updated_cb,
 		                                    subscription, NULL);
@@ -814,6 +857,10 @@ tracker_notifier_signal_subscribe (TrackerNotifier *notifier,
 	                     GUINT_TO_POINTER (subscription->handler_id),
 	                     subscription);
 
+	g_free (dbus_name);
+	g_free (dbus_path);
+	g_free (full_graph);
+
 	return subscription->handler_id;
 }
 
@@ -858,8 +905,6 @@ _tracker_notifier_get_connection (TrackerNotifier *notifier)
  * Returns the event type.
  *
  * Returns: The event type
- *
- * Since: 1.12
  **/
 TrackerNotifierEventType
 tracker_notifier_event_get_event_type (TrackerNotifierEvent *event)
@@ -876,8 +921,6 @@ tracker_notifier_event_get_event_type (TrackerNotifierEvent *event)
  * which is used as efficient internal identifier for the resource.
  *
  * Returns: the resource ID
- *
- * Since: 1.12
  **/
 gint64
 tracker_notifier_event_get_id (TrackerNotifierEvent *event)
@@ -897,8 +940,6 @@ tracker_notifier_event_get_id (TrackerNotifierEvent *event)
  * notified upon, typically of the form "urn:uuid:...".
  *
  * Returns: The element URN
- *
- * Since: 1.12
  **/
 const gchar *
 tracker_notifier_event_get_urn (TrackerNotifierEvent *event)
