@@ -225,13 +225,11 @@ set_property (GObject      *object,
 
 /**
  * tracker_resource_new:
- * @identifier: A string containing a URI
+ * @identifier: (nullable): A string containing a URI, or %NULL.
  *
  * Creates a TrackerResource instance.
  *
  * Returns: a newly created #TrackerResource. Free with g_object_unref() when done
- *
- * Since: 1.10
  */
 TrackerResource *
 tracker_resource_new (const char *identifier)
@@ -267,8 +265,6 @@ tracker_resource_new (const char *identifier)
  * You can pass any kind of GValue for @value, but serialization functions will
  * normally only be able to serialize URIs/relationships and fundamental value
  * types (string, int, etc.).
- *
- * Since: 1.10
  */
 void
 tracker_resource_set_gvalue (TrackerResource *self,
@@ -366,8 +362,6 @@ validate_pointer (const void *pointer,
  * @value: the property object
  *
  * Sets a single-valued boolean object.
- *
- * Since: 1.10
  */
 SET_PROPERTY_FOR_GTYPE (tracker_resource_set_boolean, gboolean, G_TYPE_BOOLEAN, g_value_set_boolean, validate_boolean);
 
@@ -378,8 +372,6 @@ SET_PROPERTY_FOR_GTYPE (tracker_resource_set_boolean, gboolean, G_TYPE_BOOLEAN,
  * @value: the property object
  *
  * Sets a single-valued double object.
- *
- * Since: 1.10
  */
 SET_PROPERTY_FOR_GTYPE (tracker_resource_set_double, double, G_TYPE_DOUBLE, g_value_set_double, validate_double);
 
@@ -390,8 +382,6 @@ SET_PROPERTY_FOR_GTYPE (tracker_resource_set_double, double, G_TYPE_DOUBLE, g_va
  * @value: the property object
  *
  * Sets a single-valued integer object.
- *
- * Since: 1.10
  */
 SET_PROPERTY_FOR_GTYPE (tracker_resource_set_int, int, G_TYPE_INT, g_value_set_int, validate_int);
 
@@ -402,8 +392,6 @@ SET_PROPERTY_FOR_GTYPE (tracker_resource_set_int, int, G_TYPE_INT, g_value_set_i
  * @value: the property object
  *
  * Sets a single-valued integer object.
- *
- * Since: 1.10
  */
 SET_PROPERTY_FOR_GTYPE (tracker_resource_set_int64, gint64, G_TYPE_INT64, g_value_set_int64, validate_int64);
 
@@ -417,8 +405,6 @@ SET_PROPERTY_FOR_GTYPE (tracker_resource_set_int64, gint64, G_TYPE_INT64, g_valu
  * function produces similar RDF to tracker_resource_set_uri(),
  * although in this function the URI will depend on the identifier
  * set on @resource.
- *
- * Since: 1.10
  */
 SET_PROPERTY_FOR_GTYPE (tracker_resource_set_relation, TrackerResource *, TRACKER_TYPE_RESOURCE, g_value_set_object, validate_pointer);
 
@@ -432,8 +418,6 @@ SET_PROPERTY_FOR_GTYPE (tracker_resource_set_relation, TrackerResource *, TRACKE
  * function produces similar RDF to tracker_resource_set_uri(),
  * although in this function the URI will depend on the identifier
  * set on @resource. This function takes ownership of @resource.
- *
- * Since: 2.0
  */
 SET_PROPERTY_FOR_GTYPE (tracker_resource_set_take_relation, TrackerResource *, TRACKER_TYPE_RESOURCE, g_value_take_object, validate_pointer);
 
@@ -444,8 +428,6 @@ SET_PROPERTY_FOR_GTYPE (tracker_resource_set_take_relation, TrackerResource *, T
  * @value: the property object
  *
  * Sets a single-valued string object.
- *
- * Since: 1.10
  */
 SET_PROPERTY_FOR_GTYPE (tracker_resource_set_string, const char *, G_TYPE_STRING, g_value_set_string, validate_pointer);
 
@@ -458,8 +440,6 @@ SET_PROPERTY_FOR_GTYPE (tracker_resource_set_string, const char *, G_TYPE_STRING
  * Sets a single-valued resource object as a string URI. This function
  * produces similar RDF to tracker_resource_set_relation(), although
  * it requires that the URI is previously known.
- *
- * Since: 1.10
  */
 SET_PROPERTY_FOR_GTYPE (tracker_resource_set_uri, const char *, TRACKER_TYPE_URI, g_value_set_string, validate_pointer);
 
@@ -474,8 +454,6 @@ SET_PROPERTY_FOR_GTYPE (tracker_resource_set_uri, const char *, TRACKER_TYPE_URI
  * You can pass any kind of GValue for @value, but serialization functions will
  * normally only be able to serialize URIs/relationships and fundamental value
  * types (string, int, etc.).
- *
- * Since: 1.10
  */
 void
 tracker_resource_add_gvalue (TrackerResource *self,
@@ -586,8 +564,6 @@ tracker_resource_add_gvalue (TrackerResource *self,
  * @value: the property object
  *
  * Adds a boolean object to a multi-valued property.
- *
- * Since: 1.10
  */
 ADD_PROPERTY_FOR_GTYPE (tracker_resource_add_boolean, gboolean, G_TYPE_BOOLEAN, g_value_set_boolean, validate_boolean);
 
@@ -598,8 +574,6 @@ ADD_PROPERTY_FOR_GTYPE (tracker_resource_add_boolean, gboolean, G_TYPE_BOOLEAN,
  * @value: the property object
  *
  * Adds a double object to a multi-valued property.
- *
- * Since: 1.10
  */
 ADD_PROPERTY_FOR_GTYPE (tracker_resource_add_double, double, G_TYPE_DOUBLE, g_value_set_double, validate_double);
 
@@ -610,8 +584,6 @@ ADD_PROPERTY_FOR_GTYPE (tracker_resource_add_double, double, G_TYPE_DOUBLE, g_va
  * @value: the property object
  *
  * Adds an integer object to a multi-valued property.
- *
- * Since: 1.10
  */
 ADD_PROPERTY_FOR_GTYPE (tracker_resource_add_int, int, G_TYPE_INT, g_value_set_int, validate_int);
 
@@ -622,8 +594,6 @@ ADD_PROPERTY_FOR_GTYPE (tracker_resource_add_int, int, G_TYPE_INT, g_value_set_i
  * @value: the property object
  *
  * Adds an integer object to a multi-valued property.
- *
- * Since: 1.10
  */
 ADD_PROPERTY_FOR_GTYPE (tracker_resource_add_int64, gint64, G_TYPE_INT64, g_value_set_int64, validate_int64);
 
@@ -637,8 +607,6 @@ ADD_PROPERTY_FOR_GTYPE (tracker_resource_add_int64, gint64, G_TYPE_INT64, g_valu
  * function produces similar RDF to tracker_resource_add_uri(),
  * although in this function the URI will depend on the identifier
  * set on @resource.
- *
- * Since: 1.10
  */
 ADD_PROPERTY_FOR_GTYPE (tracker_resource_add_relation, TrackerResource *, TRACKER_TYPE_RESOURCE, g_value_set_object, validate_pointer);
 
@@ -652,8 +620,6 @@ ADD_PROPERTY_FOR_GTYPE (tracker_resource_add_relation, TrackerResource *, TRACKE
  * function produces similar RDF to tracker_resource_add_uri(),
  * although in this function the URI will depend on the identifier
  * set on @resource. This function takes ownership of @resource.
- *
- * Since: 2.0
  */
 ADD_PROPERTY_FOR_GTYPE (tracker_resource_add_take_relation, TrackerResource *, TRACKER_TYPE_RESOURCE, g_value_take_object, validate_pointer);
 
@@ -665,8 +631,6 @@ ADD_PROPERTY_FOR_GTYPE (tracker_resource_add_take_relation, TrackerResource *, T
  * @value: the property object
  *
  * Adds a string object to a multi-valued property.
- *
- * Since: 1.10
  */
 ADD_PROPERTY_FOR_GTYPE (tracker_resource_add_string, const char *, G_TYPE_STRING, g_value_set_string, validate_pointer);
 
@@ -679,8 +643,6 @@ ADD_PROPERTY_FOR_GTYPE (tracker_resource_add_string, const char *, G_TYPE_STRING
  * Adds a resource object to a multi-valued property. This function
  * produces similar RDF to tracker_resource_add_relation(), although
  * it requires that the URI is previously known.
- *
- * Since: 1.10
  */
 ADD_PROPERTY_FOR_GTYPE (tracker_resource_add_uri, const char *, TRACKER_TYPE_URI, g_value_set_string, validate_pointer);
 
@@ -694,8 +656,6 @@ ADD_PROPERTY_FOR_GTYPE (tracker_resource_add_uri, const char *, TRACKER_TYPE_URI
  *
  * Returns: (transfer container) (element-type GValue): a #GList of #GValue
  * instances. The list should be freed with g_list_free()
- *
- * Since: 1.10
  */
 GList *tracker_resource_get_values (TrackerResource *self,
                                     const char      *property_uri)
@@ -771,8 +731,6 @@ GList *tracker_resource_get_values (TrackerResource *self,
  * Returns the first boolean object previously assigned to a property.
  *
  * Returns: the first boolean object
- *
- * Since: 1.10
  */
 GET_PROPERTY_FOR_GTYPE (tracker_resource_get_first_boolean, gboolean, G_TYPE_BOOLEAN, g_value_get_boolean, FALSE);
 
@@ -784,8 +742,6 @@ GET_PROPERTY_FOR_GTYPE (tracker_resource_get_first_boolean, gboolean, G_TYPE_BOO
  * Returns the first double object previously assigned to a property.
  *
  * Returns: the first double object
- *
- * Since: 1.10
  */
 GET_PROPERTY_FOR_GTYPE (tracker_resource_get_first_double, double, G_TYPE_DOUBLE, g_value_get_double, 0.0);
 
@@ -797,8 +753,6 @@ GET_PROPERTY_FOR_GTYPE (tracker_resource_get_first_double, double, G_TYPE_DOUBLE
  * Returns the first integer object previously assigned to a property.
  *
  * Returns: the first integer object
- *
- * Since: 1.10
  */
 GET_PROPERTY_FOR_GTYPE (tracker_resource_get_first_int, int, G_TYPE_INT, g_value_get_int, 0);
 
@@ -810,8 +764,6 @@ GET_PROPERTY_FOR_GTYPE (tracker_resource_get_first_int, int, G_TYPE_INT, g_value
  * Returns the first integer object previously assigned to a property.
  *
  * Returns: the first integer object
- *
- * Since: 1.10
  */
 GET_PROPERTY_FOR_GTYPE (tracker_resource_get_first_int64, gint64, G_TYPE_INT64, g_value_get_int64, 0);
 
@@ -823,8 +775,6 @@ GET_PROPERTY_FOR_GTYPE (tracker_resource_get_first_int64, gint64, G_TYPE_INT64,
  * Returns the first resource object previously assigned to a property.
  *
  * Returns: (transfer none): the first resource object
- *
- * Since: 1.10
  */
 GET_PROPERTY_FOR_GTYPE (tracker_resource_get_first_relation, TrackerResource *, TRACKER_TYPE_RESOURCE, g_value_get_object, NULL);
 
@@ -836,8 +786,6 @@ GET_PROPERTY_FOR_GTYPE (tracker_resource_get_first_relation, TrackerResource *,
  * Returns the first string object previously assigned to a property.
  *
  * Returns: the first string object
- *
- * Since: 1.10
  */
 GET_PROPERTY_FOR_GTYPE (tracker_resource_get_first_string, const char *, G_TYPE_STRING, g_value_get_string, NULL);
 
@@ -849,8 +797,6 @@ GET_PROPERTY_FOR_GTYPE (tracker_resource_get_first_string, const char *, G_TYPE_
  * Returns the first resource object previously assigned to a property.
  *
  * Returns: the first resource object as an URI.
- *
- * Since: 1.10
  */
 GET_PROPERTY_FOR_GTYPE (tracker_resource_get_first_uri, const char *, TRACKER_TYPE_URI, g_value_get_string, NULL);
 
@@ -864,8 +810,6 @@ GET_PROPERTY_FOR_GTYPE (tracker_resource_get_first_uri, const char *, TRACKER_TY
  * SPARQL blank node identifier, such as "_:123".
  *
  * Returns: a string owned by the resource
- *
- * Since: 1.10
  */
 const char *
 tracker_resource_get_identifier (TrackerResource *self)
@@ -891,8 +835,6 @@ tracker_resource_get_identifier (TrackerResource *self)
  *
  * If the identifier is set to NULL, a SPARQL blank node identifier such as
  * "_:123" is assigned to the resource.
- *
- * Since: 1.10
  */
 void
 tracker_resource_set_identifier (TrackerResource *self,
@@ -956,8 +898,6 @@ tracker_resource_identifier_compare_func (TrackerResource *resource,
  * thing.
  *
  * Returns: 0 if the identifiers are the same, -1 or +1 otherwise
- *
- * Since: 1.10
  */
 gint
 tracker_resource_compare (TrackerResource *a,
@@ -1308,8 +1248,6 @@ generate_turtle (TrackerResource    *resource,
  * usual prefixes for all of the ontologies that Tracker ships with by default.
  *
  * Returns: a newly-allocated string
- *
- * Since: 1.10
  */
 char *
 tracker_resource_print_turtle (TrackerResource         *self,
@@ -1565,8 +1503,6 @@ generate_sparql_insert_pattern (TrackerResource    *resource,
  * usual prefixes for all of the ontologies that Tracker ships with by default.
  *
  * Returns: a newly-allocated string containing a SPARQL update command.
- *
- * Since: 1.10
  */
 char *
 tracker_resource_print_sparql_update (TrackerResource         *resource,
@@ -1755,8 +1691,6 @@ generate_jsonld_namespace_mapping_foreach (gpointer key,
  * usual prefixes for all of the ontologies that Tracker ships with by default.
  *
  * Returns: a newly-allocated string containing JSON-LD data.
- *
- * Since: 2.0.5
  */
 char *
 tracker_resource_print_jsonld (TrackerResource         *self,
@@ -1843,8 +1777,6 @@ tracker_serialize_single_value (TrackerResource         *resource,
  *
  * Returns: (transfer full): A variant describing the resource,
  *          the reference is floating.
- *
- * Since: 3.0
  **/
 GVariant *
 tracker_resource_serialize (TrackerResource *resource)
@@ -1922,8 +1854,6 @@ tracker_resource_serialize (TrackerResource *resource)
  *
  * Returns: (transfer full): A TrackerResource, or %NULL if
  *          deserialization fails.
- *
- * Since: 3.0
  **/
 TrackerResource *
 tracker_resource_deserialize (GVariant *variant)
@@ -2034,3 +1964,24 @@ tracker_resource_deserialize (GVariant *variant)
 
 	return resource;
 }
+
+/**
+ * tracker_resource_get_property_overwrite:
+ * @resource: a #TrackerResource
+ * @property_uri: a string identifying the property to query
+ *
+ * Returns whether the prior values for this property would be deleted
+ * in the SPARQL issued by @resource.
+ *
+ * Returns: #TRUE if the property would be overwritten
+ *
+ * Since: 3.1
+ **/
+gboolean
+tracker_resource_get_property_overwrite (TrackerResource *resource,
+                                         const gchar     *property_uri)
+{
+	TrackerResourcePrivate *priv = GET_PRIVATE (resource);
+
+	return g_hash_table_contains (priv->overwrite, property_uri);
+}
